#
# PST module Margin - add margins to page
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
#
#
#
#
#
# 
# options:
# l|left = <unit>
# r|right = <unit>
# t|top = <unit>
# b|bottom = <unit>
#     how big should the apropriate margin be


package Margin;
use OM;
use sizes;
@ISA = qw(OM);
use strict;



sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = OM->new;

	$self->name("Margin");
	$self->{LEFT} = 0;
	$self->{RIGHT} = 0;
	$self->{TOP} = 0;
	$self->{BOTTOM} = 0;

	return bless($self,$class);
}


sub parse_options {
	my $self = shift;
	my $str = "";

	$self->log(4,"parsing options");
	while (my $s = shift) { $str .= $s }
	foreach  (split /;/, $str) {
		if (/^\s*(l|left)\s*\=\s*(\w+)\s*$/) {
			my $u = unit_size($2);
			$self->die($u) if $u =~ /^ERROR/;
			$self->{LEFT} = $u;
		} elsif (/^\s*(r|right)\s*\=\s*(\w+)\s*$/) {
			my $u = unit_size($2);
			$self->die($u) if $u =~ /^ERROR/;
			$self->{RIGHT} = $u;
		} elsif (/^\s*(t|top)\s*\=\s*(\w+)\s*$/) {
			my $u = unit_size($2);
			$self->die($u) if $u =~ /^ERROR/;
			$self->{TOP} = $u;
		} elsif (/^\s*(b|bottom)\s*\=\s*(\w+)\s*$/) {
			my $u = unit_size($2);
			$self->die($u) if $u =~ /^ERROR/;
			$self->{BOTTOM} = $u;
		} else {
			$self->die("bad option ($_)\n");
		}
	}
	$self->log(2,"margin sizes: left ",$self->{LEFT},", right ",$self->{RIGHT},
		", top ",$self->{TOP},", bottom ",$self->{BOTTOM});
}

sub page_size {
	my $self = shift;
	my $page = shift;

	return 0 if $page<1 or $page>$self->{PREV_PAGES};

	# add margins space
	$self->{PAGE_WIDTH}{$page} = $self->{PREV_PAGE_WIDTH}{$page} + $self->{LEFT} + $self->{RIGHT};
	$self->{PAGE_HEIGHT}{$page} = $self->{PREV_PAGE_HEIGHT}{$page} + $self->{TOP} + $self->{BOTTOM};
	return 1;
}



sub write_page_prolog {
	my $self = shift;
	my $page = shift;
	my $f = $self->{OUTPUT};

	if ($self->{LEFT} || $self->{BOTTOM}) {
		# just move the page a bit
		$self->log(4,"writing page $page prolog (needs shifting)");
		print $f "userdict/PSTdict save put\nPSTmatrix setmatrix\n",
			$self->{LEFT}," ",$self->{BOTTOM}," translate\n",
			"userdict/PSTmatrix matrix currentmatrix put\n",
	                #"userdict/PSTclip{0 0 moveto\n",
	                #$pw," 0 rlineto 0 ",$ph," rlineto -",$pw," 0 rlineto\n",
	                #"closepath}put initclip\n",
	                "PSTxform concat\n";

	}
}


sub write_page_epilog {
	my $self = shift;
	my $page = shift;
	my $f = $self->{OUTPUT};

	if ($self->{LEFT} || $self->{BOTTOM}) {
		$self->log(4,"writing page $page epilog");
		print $f "PSTdict restore\n";
	}
}

1;  # no init
