#
# PST module PageNumber - put numbers on pages
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
# 
# options:
# x = <unit>
# y = <unit>   where to write?
# fn|font_name = <string>     default is Times-Roman
# fs|font_size = <int>        default is 10
# t|text = "<string>"         replace character '%' with current page number and write the whole string
#
#


package PageNum;
use OM;
use sizes;
@ISA = qw(OM);
use strict;



sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = OM->new;

	$self->name("PageNum");
	$self->{POS_X} = undef;
	$self->{POS_Y} = undef;
	$self->{FONT_NAME} = "Times-Roman";  # some defaults
	$self->{FONT_SIZE} = 10;
	$self->{TEXT} = "Page %";

	return bless($self,$class);
}

sub parse_options {
	my $self= shift;
	my $str = "";

	$self->log(4,"parsing options");
	while (my $s = shift) { $str .= $s }
	foreach  (split /;/, $str) {
		if (/^\s*x\s*\=\s*(\w+)\s*$/) {
			my $u = unit_size($1);
			$self->die($u) if $u =~ /^ERROR/;
			$self->{POS_X} = $u;
		} elsif (/^\s*y\s*\=\s*(\w+)\s*$/) {
			my $u = unit_size($1);
			$self->die($u) if $u =~ /^ERROR/;
			$self->{POS_Y} = $u;
		} elsif (/^\s*(fn|font_name)\s*\=\s*(\w+)\s*$/) {
			$self->{FONT_NAME} = $2;
		} elsif (/^\s*(fs|font_size)\s*\=\s*(\d+)\s*$/) {
			$self->{FONT_SIZE} = $2;
		} elsif (/^\s*(t|text)\s*\=\s*"([^"]*)"\s*$/) {
			$self->{TEXT} = $2;
		} else {
			$self->die("bad option ($_)\n");
		}
	}
	if (!defined($self->{POS_X}) || !defined($self->{POS_Y})) {
		$self->die("I need some options\n");
	}
	$self->log(2,"string \"",$self->{TEXT},"\" on (",$self->{POS_X},",",$self->{POS_Y},")",
		" with font ",$self->{FONT_NAME}," size ",$self->{FONT_SIZE});
}


sub write_page_prolog {
	my $self = shift;
	my $page = shift;
	my $f = $self->{OUTPUT};

	$self->log(4,"writing page $page prolog");
	print $f
		"userdict/PSTdict save put\n",
		"/showpage{}def/copypage{}def/erasepage{}def\n";  # prevent these command from working
}

sub write_page_epilog {
	my $self = shift;
	my $page = shift;
	my $f = $self->{OUTPUT};
	my $t = $self->{TEXT};
	$t =~ s/%/$page/g;

	$self->log(4,"writing page $page epilog (printing commands)");
	print $f
		"PSTmatrix setmatrix\n",
		"gsave\n",
		"/",$self->{FONT_NAME}," findfont ",$self->{FONT_SIZE}," scalefont setfont\n", # set font
		$self->{POS_X}," ",$self->{POS_Y}," moveto\n",  # goto location
		"($t) show\n",  # write the string
		"grestore\n",
                "userdict/PSTmatrix matrix currentmatrix put\n",
                #"userdict/PSTclip{0 0 moveto\n",
                #$pw," 0 rlineto 0 ",$ph," rlineto -",$pw," 0 rlineto\n",
                #"closepath}put initclip\n",
                "PSTxform concat\n",
		"PSTdict restore showpage\n"; # restore state and dump page
}


1;  # no init
